@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2011 The OpenLDAP Foundation.
+ * Copyright 1998-2013 The OpenLDAP Foundation.
  * Portions Copyright 1998-2003 Kurt D. Zeilenga.
  * Portions Copyright 1998-2001 Net Boolean Incorporated.
  * All rights reserved.
@@ -260,9 +260,7 @@ main( int argc, char **argv )
 
 	free( bvalue.bv_val );
 
-	tool_unbind( ld );
-	tool_destroy();
-	return rc;
+	tool_exit( ld, rc );
 }
 
 
