@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2011 The OpenLDAP Foundation.
+ * Copyright 1998-2013 The OpenLDAP Foundation.
  * Portions Copyright 1998-2003 Kurt D. Zeilenga.
  * All rights reserved.
  *
@@ -218,9 +218,7 @@ main( int argc, char **argv )
 			fclose( fp );
 	}
 
-	tool_unbind( ld );
-	tool_destroy();
-    return retval;
+	tool_exit( ld, retval );
 }
 
 
