@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2011 The OpenLDAP Foundation.
+ * Copyright 1998-2013 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -74,6 +74,8 @@ static int option_count = 1;
 
 static int msad_range_hack = 0;
 
+static int ad_count;
+
 static Attr_option *ad_find_option_definition( const char *opt, int optlen );
 
 int ad_keystring(
@@ -382,6 +384,9 @@ done:;
 		d2->ad_flags = desc.ad_flags;
 		d2->ad_cname.bv_len = desc.ad_type->sat_cname.bv_len;
 		d2->ad_tags.bv_len = desc.ad_tags.bv_len;
+		ldap_pvt_thread_mutex_lock( &ad_index_mutex );
+		d2->ad_index = ++ad_count;
+		ldap_pvt_thread_mutex_unlock( &ad_index_mutex );
 
 		if (dlen == 0) {
 			d2->ad_cname.bv_val = d2->ad_type->sat_cname.bv_val;
@@ -770,6 +775,7 @@ int slap_bv2undef_ad(
 
 		/* shouldn't we protect this for concurrency? */
 		desc->ad_type = at;
+		desc->ad_index = 0;
 		ldap_pvt_thread_mutex_lock( &ad_undef_mutex );
 		desc->ad_next = desc->ad_type->sat_ad;
 		desc->ad_type->sat_ad = desc;
