@@ -1,8 +1,8 @@
 /* init.c - initialize ldap backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-dnssrv/init.c,v 1.29.2.6 2010/04/13 20:23:26 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2010 The OpenLDAP Foundation.
+ * Copyright 2000-2013 The OpenLDAP Foundation.
  * Portions Copyright 2000-2003 Kurt D. Zeilenga.
  * All rights reserved.
  *
@@ -49,7 +49,7 @@ dnssrv_back_initialize(
 
 	bi->bi_db_init = 0;
 	bi->bi_db_destroy = 0;
-	bi->bi_db_config = dnssrv_back_db_config;
+	bi->bi_db_config = 0 /* dnssrv_back_db_config */;
 	bi->bi_db_open = 0;
 	bi->bi_db_close = 0;
 
