@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2011 The OpenLDAP Foundation.
+ * Copyright 1998-2013 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -21,11 +21,13 @@ LDAP_BEGIN_DECL
 extern BI_init		passwd_back_initialize;
 extern BI_open		passwd_back_open;
 extern BI_destroy	passwd_back_destroy;
-extern BI_db_config	passwd_back_db_config;
 extern BI_op_search	passwd_back_search;
 
+extern int passwd_back_init_cf( BackendInfo *bi );
+
 extern AttributeDescription	*ad_sn;
 extern AttributeDescription	*ad_desc;
+
 LDAP_END_DECL
 
 #endif /* PROTO_PASSWD_H */
