@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2011 The OpenLDAP Foundation.
+ * Copyright 1999-2013 The OpenLDAP Foundation.
  * Portions Copyright 1999 John C. Quillan.
  * Portions Copyright 2002 myinternet Limited.
  * All rights reserved.
@@ -46,6 +46,10 @@ extern ldap_pvt_thread_mutex_t  perl_interpreter_mutex;
 /* All the old style variables are prefixed with PL_ now */
 # define errgv	PL_errgv
 # define na	PL_na
+#else
+# define call_method(m, f)	perl_call_method(m, f)
+# define eval_pv(m, f)	perl_eval_pv(m, f)
+# define ERRSV	GvSV(errgv)
 #endif
 
 #if defined( HAVE_WIN32_ASPERL ) || defined( USE_ITHREADS )
@@ -57,13 +61,16 @@ extern ldap_pvt_thread_mutex_t  perl_interpreter_mutex;
 # define PERL_INTERPRETER			perl_interpreter
 # define PERL_BACK_XS_INIT_PARAMS		void
 # define PERL_BACK_BOOT_DYNALOADER_PARAMS	CV *cv
+# define PERL_SET_CONTEXT(i)
 #endif
 
 extern PerlInterpreter *PERL_INTERPRETER;
 
 
 typedef struct perl_backend_instance {
-	char	*pb_module_name;
+	char *pb_module_name;
+	BerVarray pb_module_path;
+	BerVarray pb_module_config;
 	SV	*pb_obj_ref;
 	int	pb_filter_search_results;
 } PerlBackend;
