@@ -1,7 +1,7 @@
 /* operational.c - routines to deal with on-the-fly operational attrs */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2005 The OpenLDAP Foundation.
+ * Copyright 2001-2013 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,9 +29,9 @@ slap_operational_subschemaSubentry( Backend *be )
 	/* The backend wants to take care of it */
 	if ( be && !SLAP_FRONTEND(be) && be->be_schemadn.bv_val ) return NULL;
 
-	a = ch_malloc( sizeof( Attribute ) );
-	a->a_desc = slap_schema.si_ad_subschemaSubentry;
+	a = attr_alloc( slap_schema.si_ad_subschemaSubentry );
 
+	a->a_numvals = 1;
 	a->a_vals = ch_malloc( 2 * sizeof( struct berval ) );
 	ber_dupbv( a->a_vals, &frontendDB->be_schemadn );
 	a->a_vals[1].bv_len = 0;
@@ -42,9 +42,6 @@ slap_operational_subschemaSubentry( Backend *be )
 	a->a_nvals[1].bv_len = 0;
 	a->a_nvals[1].bv_val = NULL;
 
-	a->a_next = NULL;
-	a->a_flags = 0;
-
 	return a;
 }
 
@@ -57,9 +54,9 @@ slap_operational_entryDN( Entry *e )
 	assert( !BER_BVISNULL( &e->e_name ) );
 	assert( !BER_BVISNULL( &e->e_nname ) );
 
-	a = ch_malloc( sizeof( Attribute ) );
-	a->a_desc = slap_schema.si_ad_entryDN;
+	a = attr_alloc( slap_schema.si_ad_entryDN );
 
+	a->a_numvals = 1;
 	a->a_vals = ch_malloc( 2 * sizeof( struct berval ) );
 	ber_dupbv( &a->a_vals[ 0 ], &e->e_name );
 	BER_BVZERO( &a->a_vals[ 1 ] );
@@ -68,9 +65,6 @@ slap_operational_entryDN( Entry *e )
 	ber_dupbv( &a->a_nvals[ 0 ], &e->e_nname );
 	BER_BVZERO( &a->a_nvals[ 1 ] );
 
-	a->a_next = NULL;
-	a->a_flags = 0;
-
 	return a;
 }
 
@@ -82,8 +76,8 @@ slap_operational_hasSubordinate( int hs )
 
 	val = hs ? slap_true_bv : slap_false_bv;
 
-	a = ch_malloc( sizeof( Attribute ) );
-	a->a_desc = slap_schema.si_ad_hasSubordinates;
+	a = attr_alloc( slap_schema.si_ad_hasSubordinates );
+	a->a_numvals = 1;
 	a->a_vals = ch_malloc( 2 * sizeof( struct berval ) );
 
 	ber_dupbv( &a->a_vals[0], &val );
@@ -91,9 +85,6 @@ slap_operational_hasSubordinate( int hs )
 
 	a->a_nvals = a->a_vals;
 
-	a->a_next = NULL;
-	a->a_flags = 0;
-
 	return a;
 }
 
