@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2011 The OpenLDAP Foundation.
+ * Copyright 1999-2013 The OpenLDAP Foundation.
  * Portions Copyright 1999-2003 Howard Chu.
  * Portions Copyright 2000-2003 Pierangelo Masarati.
  * All rights reserved.
@@ -209,7 +209,7 @@ rwm_map_attrnames(
 	}
 
 	assert( i > 0 || x > 0 );
-	*anp = op->o_tmpalloc( ( i + x + 1 )* sizeof( AttributeName ),
+	*anp = op->o_tmpcalloc( ( i + x + 1 ), sizeof( AttributeName ),
 		op->o_tmpmemctx );
 	if ( *anp == NULL ) {
 		return LDAP_NO_MEMORY;
