@@ -1,8 +1,8 @@
 /* referral.c - muck with referrals */
-/* $OpenLDAP: pkg/ldap/servers/slapd/referral.c,v 1.23.2.1 2005/01/20 18:03:55 kurt Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2013 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -108,12 +108,7 @@ static char * referral_dn_muck(
 			}
 
 			muck.bv_len = ntargetDN.bv_len + nrefDN.bv_len - nbaseDN.bv_len;
-			muck.bv_val = SLAP_MALLOC( muck.bv_len + 1 );
-			if( muck.bv_val == NULL ) {
-				Debug( LDAP_DEBUG_ANY,
-					"referral_dn_muck: SLAP_MALLOC failed\n", 0, 0, 0 );
-				return NULL;
-			}
+			muck.bv_val = ch_malloc( muck.bv_len + 1 );
 
 			strncpy( muck.bv_val, ntargetDN.bv_val,
 				ntargetDN.bv_len-nbaseDN.bv_len );
@@ -145,7 +140,7 @@ int validate_global_referral( const char *url )
 	int rc;
 	LDAPURLDesc *lurl;
 
-	rc = ldap_url_parse_ext( url, &lurl );
+	rc = ldap_url_parse_ext( url, &lurl, LDAP_PVT_URL_PARSE_NONE );
 
 	switch( rc ) {
 	case LDAP_URL_SUCCESS:
@@ -153,6 +148,7 @@ int validate_global_referral( const char *url )
 
 	case LDAP_URL_ERR_BADSCHEME:
 		/* not LDAP hence valid */
+		Debug( LDAP_DEBUG_CONFIG, "referral \"%s\": not LDAP.\n", url, 0, 0 );
 		return 0;
 
 	default:
@@ -216,19 +212,14 @@ BerVarray referral_rewrite(
 		return NULL;
 	}
 
-	refs = SLAP_MALLOC( ( i + 1 ) * sizeof( struct berval ) );
-	if ( refs == NULL ) {
-		Debug( LDAP_DEBUG_ANY,
-			"referral_rewrite: SLAP_MALLOC failed\n", 0, 0, 0 );
-		return NULL;
-	}
+	refs = ch_malloc( ( i + 1 ) * sizeof( struct berval ) );
 
 	for ( iv = in, jv = refs; !BER_BVISNULL( iv ); iv++ ) {
 		LDAPURLDesc	*url;
 		char		*dn;
 		int		rc;
 		
-		rc = ldap_url_parse_ext( iv->bv_val, &url );
+		rc = ldap_url_parse_ext( iv->bv_val, &url, LDAP_PVT_URL_PARSE_NONE );
 		if ( rc == LDAP_URL_ERR_BADSCHEME ) {
 			ber_dupbv( jv++, iv );
 			continue;
@@ -291,12 +282,7 @@ BerVarray get_entry_referrals(
 
 	if( i < 1 ) return NULL;
 
-	refs = SLAP_MALLOC( (i + 1) * sizeof(struct berval));
-	if( refs == NULL ) {
-		Debug( LDAP_DEBUG_ANY,
-			"get_entry_referrals: SLAP_MALLOC failed\n", 0, 0, 0 );
-		return NULL;
-	}
+	refs = ch_malloc( (i + 1) * sizeof(struct berval));
 
 	for( iv=attr->a_vals, jv=refs; iv->bv_val != NULL; iv++ ) {
 		unsigned k;
