@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2004-2011 The OpenLDAP Foundation.
+ * Copyright 2004-2013 The OpenLDAP Foundation.
  * Portions Copyright 2004 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -38,6 +38,10 @@
 
 #include "slapcommon.h"
 
+#ifndef S_IWRITE
+#define S_IWRITE	S_IWUSR
+#endif
+
 static int
 test_file( const char *fname, const char *ftype )
 {
